package com.intuit.payments.qa.tests;

import java.math.BigDecimal;
import java.sql.Connection;
import java.sql.Statement;

import org.dbunit.assertion.DbUnitAssert;
import org.dbunit.database.DatabaseConfig;
import org.dbunit.dataset.CompositeDataSet;
import org.dbunit.dataset.IDataSet;
import org.dbunit.dataset.ITable;
import org.dbunit.operation.DatabaseOperation;
import org.testng.annotations.DataProvider;
import org.testng.annotations.Test;

import com.intuit.payments.qe.common.DirectoryXmlDataProvider;
import com.intuit.payments.qe.common.OasisDBConnectTestCase;
import com.intuit.payments.qe.common.RandomHelper;
import com.intuit.tame.db.DBUnitTestUtilities;

public final class NACHATest extends OasisDBConnectTestCase {
    private IDataSet dataSet;
    private Tester[] testers;

    public ExceptionSettlementTest() throws Exception {
        setupDatabaseBeforeClass();
    }

    @DataProvider(name = "onefile")
    public Object[][] onefile() {
        return new Object[][] {
                { "positive-fees.xml" },
                {"negative-discount.xml"}
        };
    }

    @Test(enabled = true, dataProvider = "scenario_dir", dataProviderClass = com.intuit.payments.qe.common.DirectoryXmlDataProvider.class)
    //@Test(enabled = true, dataProvider = "onefile")
    protected void testAllScenarios(String xmlFile) throws Exception {
        genericSettlementAndBillingBanControlTest(xmlFile);
    }

    // expected known
    // setup preconditions
    // do exercise
    // validate

    public void setupDatabaseConnection() {
        connection.getConfig().setProperty(DatabaseConfig.PROPERTY_DATATYPE_FACTORY, new CustomDataTypeFactory());
    }

    public IDataSet getDataSet(String xmlFile) throws Exception {
        return DBUnitTestUtilities.loadXmlDataSet("/" + DirectoryXmlDataProvider.scenario_directory + "/" + xmlFile);
    }

    public void printStartMessage(String xmlFile) {
        System.out.println("\nScenario: \t\t" + xmlFile.substring(0, xmlFile.length()-4));
    }

    private String getObjectChangeId() {
        String retval = RandomHelper.generateRandomSequence(22);
        System.out.println("Object change id: \t" + retval);
        return retval;
    }

    private void setupTesters() {
        testers = new Tester[] {
                new NachaBanControlTester()
        };
        Tester.Utils.setupAll(testers, connection, dataSet);
    }

    public BigDecimal genericNachaAndBanControlTest(String xmlFile) throws Exception {
        try {
            // Setup
            printStartMessage(xmlFile);
            setupDatabaseConnection();
            dataSet = getDataSet(xmlFile);
            setupTesters();
            String object_change_id = getObjectChangeId();
            setupDatabase(object_change_id);
            this.turnOffFunding();

            // exercise
            WebService.call(connection, xmlFile);
            WaitUtility.waitForAchBanControlProcessed(connection, dataSet, object_change_id);
            BigDecimal exceptionCase = ExceptionUtility.getExceptionCase(connection, dataSet, object_change_id);
            WaitUtility.waitForEnvelopeMac(connection, exceptionCase);

            // verify
            DbUnitAssert dbUnitAssert = new DbUnitAssert();

            for (Tester test : testers) {
                if (test instanceof AchBanControlTester)
                    test.verify(object_change_id, dbUnitAssert);
                else
                    test.verify(exceptionCase, dbUnitAssert);
            }

            return exceptionCase;
        } catch (Exception e) {
            System.out.println(xmlFile + " failed.\n");
            throw (e);
        } finally {
            this.turnOnFunding();
        }
    }

    private void setupDatabase(String object_change_id) throws Exception {
        DatabaseOperation.INSERT.execute(connection, prepareInputDataSet(object_change_id));
    }

    private IDataSet prepareInputDataSet(String object_change_id) throws Exception {
        //ITable achMonetaryTransactionTable = new AchMonetaryTransaction_enc(connection, dataSet, object_change_id).prepare();
        //ITable settlementMessageLogTable = new SettlementMessageLog(connection, dataSet, object_change_id).prepare();
        ITable merchantStatusHistoryTable = new Merchant_Status_History(connection, dataSet, object_change_id).prepare();

        return new CompositeDataSet(new ITable[] { achMonetaryTransactionTable, settlementMessageLogTable });
    }

    public IDataSet getDataSet() {
        return dataSet;
    }

}
