package com.intuit.payments.qa.tests;


import java.math.BigDecimal;

import org.dbunit.assertion.DbUnitAssert;
import org.dbunit.assertion.DefaultFailureHandler;
import org.dbunit.assertion.Difference;
import org.dbunit.database.IDatabaseConnection;
import org.dbunit.dataset.DefaultTable;
import org.dbunit.dataset.IDataSet;
import org.dbunit.dataset.ITable;
import org.dbunit.dataset.NoSuchColumnException;
import org.dbunit.dataset.filter.DefaultColumnFilter;
import org.testng.Assert;

public class NachaBanControlTester implements Tester {
    private IDataSet dataSet;
    private IDatabaseConnection connection;

    private ITable prepareActual(BigDecimal exceptionCase) throws Exception {
	    // first replace the exceptioncase in all sql of all actualfd_exceptiondetails rows
	    DefaultTable fd_sql_table = (DefaultTable) dataSet.getTable("actualfd_exceptiondetails");

	    for (int i = 0; i < fd_sql_table.getRowCount(); i++) {
	        String sql = Utility.replaceExceptionCaseInString((String) fd_sql_table.getValue(i, "sql"), exceptionCase);
	        fd_sql_table.setValue(i, "sql", sql);
	    }

	    DefaultTable returnTable = new DefaultTable(dataSet.getTableMetaData("sonora.fd_exceptiondetails"));
	    for (int i = 0; i < fd_sql_table.getRowCount(); i++) {
	        ITable fd_table = connection.createQueryTable("sonora.fd_exceptiondetails", (String) fd_sql_table.getValue(i, "sql"));
	        BigDecimal transaction_status_id = (BigDecimal) fd_table.getValue(0, "transaction_status_id");
	        Assert.assertTrue(transaction_status_id.compareTo(new BigDecimal("31")) == 0 || transaction_status_id.compareTo(new BigDecimal("33")) == 0);
	        returnTable.addTableRows(fd_table);
	    }

	    // filter our ach_request_id, transaction_status_id we don't compare to expected
	    return DefaultColumnFilter.excludedColumnsTable(returnTable, new String[] { "ach_request_id", "transaction_status_id" });
	}

	private ITable prepareExpected(BigDecimal exceptionCase) throws Exception {
	    DefaultTable table = (DefaultTable) dataSet.getTable("sonora.fd_exceptiondetails");
	    try {
	        // test that this row exists in the expected results
	        table.getValue(0, "s_rowid");
	        String problem_srowid_sql = Utility.replaceExceptionCaseInString((String) dataSet.getTable("problem_srowid").getValue(0, "sql"), exceptionCase);
	        String solution_srowid_sql = Utility.replaceExceptionCaseInString((String) dataSet.getTable("solution_srowid").getValue(0, "sql"), exceptionCase);

	        BigDecimal problem_srowid = (BigDecimal) connection.createQueryTable("sonora.fd_exceptiondetails", problem_srowid_sql).getValue(0, "s_rowid");
	        BigDecimal solution_srowid = (BigDecimal) connection.createQueryTable("sonora.fd_exceptiondetails", solution_srowid_sql)
	                .getValue(0, "s_rowid");

	        table.setValue(0, "s_rowid", problem_srowid);
	        table.setValue(1, "s_rowid", solution_srowid);
	        table.setValue(1, "parent_srowid", problem_srowid);
	    } catch (NoSuchColumnException e) {
	        // NOP it's not a problem solution scenario
	    }

	    // filter "type", "ach_request_id", "transaction_status_id" we won't need them for the comparison
	    return DefaultColumnFilter.excludedColumnsTable(table, new String[] { "type", "ach_request_id", "transaction_status_id" });
	}

    public void setup(IDatabaseConnection connection, IDataSet dataSet) {
        this.connection = connection;
        this.dataSet = dataSet;
    }

    public void teardown() {
        //DO NOTHING IN TEARDOWN
    }

    public void validateArguments (Object... objects) throws IllegalArgumentException {
        Tester.Utils.validateArguments(objects);
    }

    /**
     * Invoked as a test step during test setup.
     *
     * @param objects[0] the exceptionCase or ref-id used to find/prepare the tables
     *      This is a required argument and will throw an exception if it's not there.
     *
     * @param objects[1] the DbUnitAssert object
     *      This is a required argument and will throw an exception if it's not there.
     *
     */

    public void verify(Object... args) throws Exception {
        Tester.Utils.validateArguments(args);

        ITable actualFdExceptionDetails = prepareActual((BigDecimal) args[0]);
        ITable expectedFdExceptionDetails = prepareExpected((BigDecimal) args[0]);
        FDExceptionDetailsFailureHandler failureHandler = new FDExceptionDetailsFailureHandler();
        ((DbUnitAssert)args[1]).assertEquals(actualFdExceptionDetails, expectedFdExceptionDetails, failureHandler);
    }

}


class NachaFailureHandler extends DefaultFailureHandler {
    @Override
    public void handle (Difference d) {
        if (d.getColumnName().equals("parent_srowid")) {
            Object expectedValue = d.getExpectedValue();
            Object actualValue = d.getActualValue();
            if (actualValue != null && actualValue.equals(expectedValue)) {
                return;
            }  else if (expectedValue != null && expectedValue.equals(actualValue)) {
                return;
            } else if (expectedValue == null && actualValue == null) {
                return;
            } else if (expectedValue == null && actualValue.equals("null")) {
                return;
            } else if (actualValue == null && expectedValue.equals("null")) {
                return;
            }
        } else {
            super.handle(d);
        }

    }
}